#Copyright 2017-2018 blacknoise.at
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
import logging
from   kivy.uix.screenmanager import ScreenManager, Screen
from   kivy.uix.gridlayout    import GridLayout
from   kivy.uix.boxlayout     import BoxLayout
from   kivy.uix.floatlayout   import FloatLayout
from   kivy.uix.button        import Button
from   kivy.uix.label         import Label
from   kivy.lang              import Builder
from   kivy.properties        import NumericProperty
from   kivy.uix.textinput     import TextInput
from   kivy.uix.behaviors     import ButtonBehavior
from   kivy.uix.image         import Image
from   kivy.uix.popup         import Popup
from   kivy.graphics          import *
from   kivy.utils             import platform
from   kivy.metrics           import dp, sp
from   kivy.config            import Config
from   kivy.uix.stencilview   import StencilView
#rom   kivy.input             import *
import webbrowser

import Version
from   bncSQLLib              import *
from   bncSQLTableKivy        import *


class BoxLayoutStencil ( BoxLayout, StencilView ):
    pass


MainScreen_kv = str ( '''
#:kivy 1.9.0
#:import hex kivy.utils.get_color_from_hex

<MainScreen>:
    id: MainScreen
    name: 'MainScreen'
    canvas.before:
        Color:
            rgb: hex('#FFFFFF')
        Rectangle:
            size: self.size
            pos:  self.pos

    BoxLayout:
        id: RootBox
        orientation: 'vertical'
        BoxLayoutStencil:
            id: MainScreenHeader
            size_hint: 1,None
            height: 0
            pos_hint: {'top':1}
            ImageButton:
                source: 'bncsql-bn-32x32.png'
                on_press: MainScreen.HomePage ( self )
                allow_stretch: True
                keep_ratio:    True
                size_hint_y:   None
                size_hint_x:   None
                height:        self.parent.height if root.width *.5 >= self.parent.height * self.image_ratio else self.parent.height / 2
                width:         self.height * self.image_ratio
            Label:
                text: ''
            ImageButton:
                source: 'view-refresh-7.png'
                on_press: MainScreen.Refresh ( self )
                allow_stretch: True
                keep_ratio:    True
                size_hint_y:   None
                size_hint_x:   None
                height:        self.parent.height #2do rm * 1
                width:         self.height * self.image_ratio
            ImageButton:
                source: 'system-run-6.png'
                on_press: MainScreen.Config ( self )
                allow_stretch: True
                keep_ratio:    True
                size_hint_y:   None
                size_hint_x:   None
                height:        self.parent.height #2do rm * 1
                width:         self.height * self.image_ratio

        BoxLayoutStencil:
            id: MainScreenQueryPrompt
            size_hint: 1,None
            height: 0
            pos_hint: {'top':1}
            Button:
                id: QueryPromptDropDownButton
                size_hint: None,1
                text: "..."
                width: MainScreen.Height
            TextInput:
                id: QueryText
                text: '' #test 'select * from bnctest'
                multiline: False
                on_text_validate: MainScreen.QueryPromptText ( self )

        GridLayout:
            id: Grid
            pos_hint: {'top':1}

''' )


class MainScreen ( Screen ):
    #logging.info ( "class MainScreen" )
    TableFokus = -1

    def __init__(self, **kwargs):
	self.name   = 'MainScreen'
	self.Height = 40 * Version.mainZoom
	Builder.load_string (  MainScreen_kv  )
	#uilder.load_file   ( 'MainScreen.kv' )
	super(MainScreen, self).__init__(**kwargs)
	#logging.info ( str ( self ) )
	SetBackground ( self, Version.mainBackground )

    def on_enter(self):
	logging.info ( str ( self ) )
	logging.info ( str ( self ) + " Version.mainLoad=" + str ( Version.mainLoad ) )

	Version.mainApp.Profile.Clean ()
	if Version.mainLoad.endswith ( '.bncsql' ):
	    Version.mainApp.Profile.Read ( os.path.join ( Version.mainHome, Version.mainLoad ) )

	if ( 1 == int ( Config.get ( 'bncsql', 'Header'      ) ) ):
	    self.ids.MainScreenHeader.height = self.Height
	else:
	    self.ids.MainScreenHeader.height = 0

	self.QueryPromptDropDownClear()
	if ( 1 == int ( Config.get ( 'bncsql', 'QueryPrompt' ) ) ):
	    self.ids.MainScreenQueryPrompt.height = self.Height
	    self.QueryPromptDropDownFill()
	else:
	    self.ids.MainScreenQueryPrompt.height = 0

	self.TableCount = int ( Config.get ( 'bncsql', 'Tables' ) )
	self.TableFokus = -1
	self.ids.Grid.clear_widgets()
	if ( 1 == self.TableCount ):
	    self.ids.Grid.cols      = 1
	    self.ids.Grid.rows      = 1
	elif ( 2 == self.TableCount ):
	    self.ids.Grid.cols      = 2
	    self.ids.Grid.rows      = 1
	elif ( 4 == self.TableCount ):
	    self.ids.Grid.cols      = 2
	    self.ids.Grid.rows      = 2
	for x in range ( self.TableCount ):
	    Table_x_ = bncSQLTableKivy();
	    self.ids.Grid.add_widget ( Table_x_ )

	self.Refresh ( self )

    def on_leave(self):
	logging.info ( str ( self ) )
	self.QueryPromptDropDownClear()

    def Pause(self, Widget):
	logging.info ( str ( self ) )

    def Resume(self, Widget):
	logging.info ( str ( self ) )
	self.Refresh ( Widget )

    def HomePage(self, args):
	logging.info ( str ( self ) + " args=" + str ( args ) )
	webbrowser.open("http://www.blacknoise.at/?bncsql")

    def Config(self, args):
	logging.debug ( str ( self ) + " args=" + str ( args ) )
	try:
	    self.parent.transition.direction = 'left'
	    self.parent.current = 'bncSQLRSyncApp'
	except:
	    logging.error ( str ( self ) )
	    pass

    def QueryPromptDropDownClear(self):
	#logging.info ( str ( self ) )
	Widget = self.ids.QueryPromptDropDownButton
	logging.info ( str ( self ) + " Widget=" + str ( Widget ) )
	try:
	    #logging.info ( str ( self ) + " Widget.dropdown=" + str ( Widget.dropdown ) )
	    Widget.dropdown.dismiss ()
	    Widget.dropdown.clear_widgets()
	    del Widget.dropdown
	except:
	    pass

    def QueryPromptDropDownFill(self):
	#logging.info ( str ( self ) )
	self.QueryPromptDropDownClear()
	self.ids.QueryPromptDropDownButton.clear_widgets() #dummy, there is no self.ids.QueryPromptDropDownButton.add_widget...
	Widget          = self.ids.QueryPromptDropDownButton
	Widget.dropdown = DropDown()
	for QueryPos in range ( len ( Version.mainApp.Profile.Querys ) ):
	    Query = Version.mainApp.Profile.Querys[ QueryPos ][0]
	    btn            = Button()
	    btn.text       = str ( Query ) #bug2 cut
	    btn.size_hint  = (None, None)
	    btn.width      = 400 * Version.mainZoom
	    btn.height     =  40 * Version.mainZoom
	    btn.bind(on_release=lambda btn: Widget.dropdown.select(btn.text))
	    Widget.dropdown.add_widget(btn)
	Widget.bind(on_release=Widget.dropdown.open)
	Widget.dropdown.auto_width = False
	Widget.dropdown.width      = 400 * Version.mainZoom
	Widget.dropdown.bind(on_select=self.QueryPromptDropDownChange)

    def TableFokusSet(self):
	if   ( 1 == self.TableCount ):
	    TableFokus = 0
	elif ( 2 == self.TableCount ):
	    if   -1 == self.TableFokus: TableFokus = 1
	    elif  1 == self.TableFokus: TableFokus = 0
	    elif  0 == self.TableFokus: TableFokus = 1
	elif ( 4 == self.TableCount ):
	    if   -1 == self.TableFokus: TableFokus = 3
	    elif  3 == self.TableFokus: TableFokus = 2
	    elif  2 == self.TableFokus: TableFokus = 1
	    elif  1 == self.TableFokus: TableFokus = 0
	    elif  0 == self.TableFokus: TableFokus = 3
	self.TableFokus = TableFokus
	logging.info ( str ( self ) + " TableFokus=" + str ( self.TableFokus ) )

    def Refresh(self, Widget):
	logging.info ( str ( self ) + " Widget=" + str ( Widget ) )
	self.TableFokus = -1
	for QueryPos in range ( len ( Version.mainApp.Profile.Querys ) ):
	    Query     = str ( Version.mainApp.Profile.Querys[ QueryPos ][0] )
	    AutoStart = int ( Version.mainApp.Profile.Querys[ QueryPos ][1] )
	    Header    = int ( Version.mainApp.Profile.Querys[ QueryPos ][2] )
	    if not AutoStart: continue
	    logging.info ( str ( self ) + " Query=" + str ( Query ) )
	    self.TableFokusSet()
	    Table = self.ids.Grid.children[self.TableFokus]
	    Table.Close ()
	    Table.DataBase   = Version.mainApp.Profile.DataBase
	    Table.HeaderShow = Header
	    Table.Query      = Query
	    Table.Draw ()

    def QueryPromptText(self, Widget):
	#logging.info ( str ( self ) + " Widget=" + str ( Widget ) )
	self.QueryPromptDropDownChange( Widget, Widget.text)

    def QueryPromptDropDownChange(self, Widget, Query):
	#logging.info ( str ( self ) + " Widget=" + str ( Widget ) )
	#logging.info ( str ( self ) + " Query="  + str ( Query  ) )
	self.TableFokusSet()
	Table = self.ids.Grid.children[self.TableFokus]
	Table.Close ()
	Table.DataBase = Version.mainApp.Profile.DataBase
	Table.Query    = Query
	Table.Draw ()
